///////////////////////////////////////////////////////////////////////////
// Copyright © 2015 Esri. All Rights Reserved.
//
// Licensed under the Apache License Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
///////////////////////////////////////////////////////////////////////////
define([

  'dojo/_base/declare',
  'dojo/_base/lang',
  'dojo/_base/html',
  'dojo/topic',
  'dojo/on',
  'dojo/query',
  './FeatureActionManager',
  './utils',
  './dijit/FeatureActionPopupMenu',
  './RelatedRecordsPopupProjector',
  "dojo/dom-attr",
  "dojo/dom",
  'esri/layers/FeatureLayer',
  "esri/dijit/InfoWindow",
  "esri/dijit/Popup",
  'jimu/dijit/Message',
  'esri/tasks/GPMessage',
  "esri/tasks/Geoprocessor",
  'esri/request',
  'esri/tasks/JobInfo'
  ], function(declare, lang, html, topic, on, query, FeatureActionManager, jimuUtils, PopupMenu,
  RelatedRecordsPopupProjector, domAttr, dom, FeatureLayer, InfoWindow, Popup,
  Message, GPMessage, Geoprocessor, esriRequest, JobInfo) {
    var instance = null;
    var clazz = declare(null, {
      mapManager: null,
      // popupUnion = {
      //   mobile: is mobile popup of map,
      //   bigScreen: is popup of map
      // };
      popupUnion: null,
      _relatedRecordsPopupProjector: null,

      constructor: function(options) {
        lang.mixin(this, options);

        this.popupMenu = PopupMenu.getInstance();
        this.isInited = false;

        this.featureActionManager = FeatureActionManager.getInstance();
        topic.subscribe("mapLoaded", lang.hitch(this, this.onMapLoadedOrChanged));
        topic.subscribe("mapChanged", lang.hitch(this, this.onMapLoadedOrChanged));
        topic.subscribe("appConfigChanged", lang.hitch(this, this._onAppConfigChanged));
        topic.subscribe("widgetsActionsRegistered", lang.hitch(this, this._onWidgetsActionsRegistered));
      },

      init: function() {
        this.popupUnion = this.mapManager.getMapInfoWindow();
        if(!this.popupUnion.bigScreen || !this.popupUnion.mobile ||
          !this.popupUnion.bigScreen.domNode || !this.popupUnion.mobile.domNode){
          return;
        }
        //this.popupUnion.bigScreen.visibleWhenEmpty = false;
        //this.popupUnion.bigScreen.hideDelay = 0;
        if(!this.isInited){
          this._createPopupMenuButton();
          this._bindSelectionChangeEvent();
          this.isInited = true;
        }
      },

      _createPopupMenuButton: function(){
        if(this.popupMenuButtonDesktop) {
          html.destroy(this.popupMenuButtonDesktop);
        }
        if(this.popupMenuButtonMobile) {
          html.destroy(this.popupMenuButtonMobile);
        }
        this.popupMenuButtonDesktop = html.create('span', {
          'class': 'popup-menu-button'
        }, query(".actionList", this.popupUnion.bigScreen.domNode)[0]);

        var mobileActionListNode =
          query("div.esriMobileInfoView.esriMobilePopupInfoView .esriMobileInfoViewItem").parent()[0];
        var mobileViewItem = html.create('div', {
            'class': 'esriMobileInfoViewItem'
          }, mobileActionListNode);
        this.popupMenuButtonMobile = html.create('span', {
          'class': 'popup-menu-button'
        }, mobileViewItem);

        on(this.popupMenuButtonMobile, 'click', lang.hitch(this, this._onPopupMenuButtonClick));
        on(this.popupMenuButtonDesktop, 'click', lang.hitch(this, this._onPopupMenuButtonClick));

        on(this.popupUnion.bigScreen, "set-features", lang.hitch(this, this._onSetFeatures));
      },

      _onSetFeatures: function(evt){
        this.selectedFeature = evt.target.getSelectedFeature();
        if(!this.selectedFeature){
          return;
        }
        var selectedFeatureLayer = this.selectedFeature.getLayer();
        if(selectedFeatureLayer.name === "TrafficCams"){
          var link = html.create("a", {
            "class": "action bridge",
            "id": "statsLink",
            "innerHTML": "Bridge Report", //text that appears in the popup for the link
            "href": "javascript: void(0);"
          }, query(".actionList", this.popupUnion.bigScreen.domNode)[0]);

          //when the link is clicked register a function that will run
          on(link, "click", lang.hitch(this, this.calculateBridgeReport));
        }else{
          var actions = query(".action.bridge", this.popupUnion.bigScreen.domNode)[0];
          html.destroy(actions);
        }
      },

      _onPopupMenuButtonClick: function(evt){
        var position = html.position(evt.target);
        this.popupMenu.show(position);
      },

      _bindSelectionChangeEvent: function(){
        on(this.popupUnion.bigScreen, "selection-change", lang.hitch(this, this._onSelectionChange));
        on(this.popupUnion.mobile, "selection-change", lang.hitch(this, this._onSelectionChange));
      },

      _onSelectionChange: function(evt){
        this.selectedFeature = evt.target.getSelectedFeature();
        if(!this.selectedFeature){
          this._disablePopupMenu();
          return;
        }
        this.initPopupMenu([this.selectedFeature]);

        var selectedFeatureLayer = this.selectedFeature.getLayer();
        var hasInfoTemplate = this.selectedFeature.infoTempalte ||
                              (selectedFeatureLayer && selectedFeatureLayer.infoTemplate);
        if(hasInfoTemplate) {
          this._createRelatedRecordsPopupProjector(this.selectedFeature);
        }
      },

      calculateBridgeReport: function() {
        var feature = this.mapManager.map.infoWindow.getSelectedFeature();
        var fs = feature.attributes.bridge_name_num;

        console.info(fs);
        console.info(feature);
        if(fs === null) {
          var message = new Message({
            message: "The selected bridge does not have a 'Bridge Name / No' value. Fill out the field and try again!",
            titleLabel: "Error"
          });
        } else if(fs === undefined) {
          var message3 = new Message({
            message: "The feature you selected is not a bridge",
            titleLabel: "Error"
          });
        } else {
          this.gpSubmit(fs);
        }
      },

      gpSubmit: function(fs) {
        this.gp = new Geoprocessor("myService");

        var params = {
          "Bridge_Name": fs
        }
        console.log(params);
        this.gp.submitJob(params, lang.hitch(this, this.jobComplete), this.jobStatus, lang.hitch(this, this.jobError));
      },

      jobComplete: function(jobInfo) {
        this.gp.getResultData(jobInfo.jobId, "pdf_export", lang.hitch(this, function(data) {
          console.log(data);

          if(data.value.url.includes("bridge_report_glulam_35521219102017.pdf")) {
            var message = new Message({
              message: "The selected bridge does not have an inspection report!",
              titleLabel: "Error"
            });
          } else {
            var message2 = new Message({
              message: "<a class='one' href='" + data.value.url + "' target='_blank'><div><img src='widgets/ReportingTool_v2.0/images/pdf.png' height='20px'><b><sup>Report Ready!</sup></b></div></a>",
              titleLabel: "Bridge Report"
            });
            //this.results.innerHTML = "<a class='one' href='" + data.value.url + "' target='_blank'><div><img src='widgets/ReportingTool_v2.0/images/pdf.png' height='20px'>&nbsp;&nbsp;<b><u><sup>Bridge Report Ready!</sup></u></b></div></a>";
          }

        }));

        console.log(jobInfo.jobStatus);
        console.log(jobInfo.results);
      },

      jobStatus: function(jobInfo) {
        console.log(jobInfo);
      },

      jobError: function(error) {
        //this.results.innerHTML = "<center><b>Report Failed...</b></center>";
      },

      _disablePopupMenu: function() {
        html.addClass(this.popupMenuButtonDesktop, 'disabled');
        html.addClass(this.popupMenuButtonMobile, 'disabled');
      },

      _enablePopupMenu: function() {
        html.removeClass(this.popupMenuButtonDesktop, 'disabled');
        html.removeClass(this.popupMenuButtonMobile, 'disabled');
      },

      // public method, can be called from outside.
      initPopupMenu: function(features){
        if(!features) {
          this._disablePopupMenu();
          this.popupMenu.setActions([]);
          return;
        }
        var featureSet = jimuUtils.toFeatureSet(features);
        this.featureActionManager.getSupportedActions(featureSet).then(lang.hitch(this, function(actions){
          var excludeActions = ['ZoomTo', 'ShowPopup', 'Flash', 'ExportToCSV',
            'ExportToFeatureCollection', 'ExportToGeoJSON', 'ShowRelatedRecords', 'SaveToMyContent'];
          var popupActions = actions.filter(lang.hitch(this, function(action){
            return excludeActions.indexOf(action.name) < 0 ;
          }));

          if(popupActions.length === 0){
            this._disablePopupMenu();
          }else{
            this._enablePopupMenu();
          }
          var menuActions = popupActions.map(lang.hitch(this, function(action){
            //action.data = jimuUtils.toFeatureSet(feature);
            action.data = featureSet;
            return action;
          }));
          this.popupMenu.setActions(menuActions);
        }));
      },

      /******************************
       * Events
       ******************************/
      onMapLoadedOrChanged: function() {
        this.isInited = false;
        this.init();
      },

      _onAppConfigChanged: function() {
        if(this.popupUnion) {
          if(this.popupUnion.bigScreen && this.popupUnion.bigScreen.hide) {
            this.popupUnion.bigScreen.hide();
            this.popupMenu.hide();
          }
          if(this.popupUnion.mobile && this.popupUnion.mobile.hide) {
            this.popupUnion.mobile.hide();
            this.popupMenu.hide();
          }
        }
      },

      _onWidgetsActionsRegistered: function(){
        //to init actions
        this.init();
      },

      /**********************************
       * Methods for show related records
       **********************************/

      _createRelatedRecordsPopupProjector: function(selectedFeature) {
        try {
          if(this._relatedRecordsPopupProjector &&
             this._relatedRecordsPopupProjector.domNode) {
            this._relatedRecordsPopupProjector.destroy();
            this._relatedRecordsPopupProjector = null;
          }
          //var refDomNode = query(".esriViewPopup", this.popupUnion.bigScreen.domNode)[0];
          this._relatedRecordsPopupProjector = new RelatedRecordsPopupProjector({
            originalFeature: selectedFeature,
            //refDomNode: refDomNode,
            popup: this.mapManager.map.infoWindow,
            popupManager: this
          });
        } catch(err) {
          console.warn(err.message);
        }
      }


    });

    clazz.getInstance = function(mapManager) {
      if (instance === null) {
        instance = new clazz({
          mapManager: mapManager
        });
      }
      return instance;
    };

    return clazz;
  });
